# -*- coding:Utf-8 -*-
# ./addons/eventscripts/_libs/python/entitylib.py
import es, vecmath, playerlib


# ==================================================
# >> EXCEPTIONS
# ==================================================
class EntityError(Exception):
    pass


# ==================================================
# >> CLASSES
# ==================================================
class EntityManager(object):
    def __init__(self, index, name = ""):
        self.index = index
        self.props = es.createentitylist(index)[index]
        self.name = str(name) if name else "_entitylib_entity_" + str(index)
        
    def __getitem__(self, item):
        returnvalue = es.getindexprop(self.index, item)
        if not returnvalue:
            for prop in self.props:
                if item.lower() in prop.lower():
                    returnvalue = es.getindexprop(self.index, prop)
                    break
        return returnvalue
        
    def __setitem__(self, item, value):
        if es.getindexprop(self.index, item) is not None:
            es.setindexprop(self.index, item, value)
        else:
            for prop in self.props:
                if item.lower() in prop.lower():
                    es.setindexprop(self.index, prop, value)
                    break
                    
    def __getattr__(self, attribute):
        returnvalue = None
        if attribute in self.__dict__:
            returnvalue = object.__getattr__(self, attribute)
        else:
            value = es.entitygetvalue(self.index, attribute)
            if value:
                returnvalue = value
        return returnvalue
        
    def __setattr__(self, attribute, value):
        if attribute == "name":
            object.__setattr__(self, attribute, str(value) if value else "_entitylib_entity_" + str(self.index))
        elif attribute in ('index', 'props'):
            object.__setattr__(self, attribute, value)
        else:
            es.entitysetvalue(self.index, attribute, value)

    def fire(self, output):
        userid = es.getuserid()
        if not userid: es.dbgmsg(1, "Entitylib: there is no users currently")
        else: es.fire(userid, self.name, output)
    

# ==================================================
# >> UTILITAIRES
# ==================================================
def getEntity(entity):
    index = getEntityIndex(entity)
    if not index:
        raise EntityError, 'Entity "%s" not found!' % entity
    return EntityManager(index)
getentity = getEntity

def getIndexList(entity = None):
    returnvalue = list()
    if not entity:
        returnvalue = es.createentityindexlist().keys()
    else:
        returnvalue = es.createentityindexlist(entity).keys()
        if not len(returnvalue):
            entities = es.createentityindexlist()
            for index in entities:
                if str(entity).lower() in entities[index]['classname']:
                    returnvalue.append(index)
    return returnvalue
getindexlist = getIndexList

def getEntityList(entity = None):
    returnvalue = list()
    for index in getIndexList(entity):
        if index:
            returnvalue.append(EntityManager(index))
    return returnvalue
getentitylist = getEntityList

def create(entity, name = ""):
    entity, name = str(entity), str(name)
    index = es.createentity(entity)
    if name: es.setentityname(index, name)
    else: es.setentityname(index, "_entitylib_entity_" + str(index))
    return EntityManager(index, name)

def exists(entity):
    return bool(getEntityIndex(entity))
    
def getEntityIndex(entity):
    returnvalue = None
    if str(entity).isdigit() and bool(es.getindexprop(entity, 'CBaseEntity.m_hOwnerEntity')):
        returnvalue = int(entity)
    else:
        for index in es.createentityindexlist():
            if str(entity).lower() == es.entitygetvalue(index, 'targetname'):
                returnvalue = index
                break
    return returnvalue
getentityindex = getEntityIndex


# ==================================================
# >> FONCTIONS
# ==================================================


def _targetlist(target):
    if isinstance(target, str):
        target = target.lower()
        if target.startswith("#"): return playerlib.getUseridList(target)
        elif target.isdigit(): useridlist = [int(target)]
        else: useridlist = [es.getuserid(target)]

    elif isinstance(target, int): useridlist = [int(target)]
    else:
        try: useridlist = map(es.getuserid, target)
        except TypeError: useridlist = []
    return [userid for userid in useridlist if es.exists("userid", userid)]

def _vectorlist(vector):
    if isinstance(vector, str):
        try: vectorlist = [vecmath.Vector(vector)]
        except Exception: vectorlist = []
    else:
        try: vectorlist = map(vecmath.Vector, vector)
        except Exception: vectorlist = []

    coordlist = []
    for vector in vectorlist:
        if len(list(vector)) == 3: coordlist.append(str(vector))
        else: es.dbgmsg(0, "Entitylib: this is not a valid vector : %s" % vector)
    return coordlist

def explosion(origin, magnitude = 200, radius = None, fireballsprite = None, rendermode = None, flags = None):
    targetlist = _targetlist(origin)
    if targetlist: vectorlist = _vectorlist(map(es.getplayerlocation, targetlist))
    else: vectorlist = _vectorlist(origin)
    if not vectorlist: es.dbgmsg(1, 'Entitylib: no vector found with parameter origin "%s"' % origin)
    else:

        userid = es.getuserid()
        if userid:
            setflag = False
            if isinstance(flags, int): setflag = flags if flags > 0 else False
            elif isinstance(flags, str):
                if flags.isdigit():
                    setflag = int(flags)
                    setflag = flags if flags > 0 else False
                else:

                    flaglist = set()
                    flagdict = {
                        "#nodamage" : 1,
                        "#repeatable" : 2,
                        "#nofireball" : 4,
                        "#nosmoke" : 8,
                        "#nodecal" : 16,
                        "#nosparks" : 32,
                        "#nosound" : 64,
                        "#randomorientation" : 128,
                        "#nofireballsmoke" : 256,
                        "#noparticles" : 512,
                        "#nodlights" : 1024,
                        "#donotclampmin" : 2048,
                        "#donotclampmax " : 4096}

                    for flag in flags.split(","):
                        flag = flag.lower()
                        if flagdict.has_key(flag): flaglist.add(flag)
                    if flaglist:
                        setflag = 0
                        for flag in flaglist: setflag |= flagdict[flag]

            entity = create("env_explosion")
            try: entity.imagnitude = int(magnitude)
            except Exception: pass
            try: entity.iradiusoverride = int(radius)
            except Exception: pass
            if fireballsprite is not None: entity.fireballsprite = fireballsprite
            if rendermode in (0, 4, 5): entity.rendermode = rendermode
            if setflag is not False: entity.fire('addoutput "spawnflags %s"' % str(setflag))

            for vector in vectorlist:
                entity.origin = " ".join(map(str, vector.split(",")))
                #es.spawnentity(entity.index)
                #teleport.target = entity.name
                #teleport.origin = " ".join(map(str, vector.split(",")))
                #teleport.fire("teleport")
                #teleport.fire("teleport " + " ".join(map(str, vector.split(","))))
                #es.setindexprop(entity.index, "CBaseEntity.m_vecOrigin", vector)
                #entity.fire('addoutput "origin %s"' % " ".join(map(str, vector.split(","))))
                entity.fire("explode")
            entity.fire("kill")
