# -*- coding:Utf-8 -*-
# ./addons/eventscripts/_libs/python/maplistlib.py
# version 1.003p

import es, os, re, psyco
gamedir = es.getString("eventscripts_gamedir")
regex = re.compile('//.*|(\r|\n|")+')
psyco.full()

class MapList(object):
    def __init__(self, path = None, validate = False):
        # set the file directory by default
        # if path is None, it will return all maps in ../GameName/maps/*
        self.path = path
        # commands will validate map(s) before running (value by default)
        # commands with parameter "validate" can override on this value
        self.validate = validate

    def __add__(self, mapname):
        # add a map in the file: myListe + mapname
        addlist(mapname, self.path, self.validate)

    def __sub__(self, mapname):
        # remove a map from the file: myListe - mapname
        remlist(mapname, self.path)

    def __len__(self):
        # get count of maps from the file: len(myListe)
        return len(getlist(self.path, self.validate))

    """ paramenter "validate" can be temporary changed """
    def __contains__(self, mapname):
        # get count of maps from the file: if mapname in myListe (>> return True/False)
        return mapname in getlist(self.path, self.validate)

    def getlist(self, validate = None):
        # get list of maps from the file: newList = myList.getlist( [,validate])
        return getlist(self.path, self.validate if validate is None else bool(validate))

    def setlist(self, maplist, validate = None):
        # save a new list of maps in the file: myList.getlist(newList [,validate])
        setlist(maplist, self.path, self.validate if validate is None else bool(validate))

    def addlist(self, mapname, validate = None):
        # same as __add__method:  myList.getlist(mapname [,validate])
        addlist(mapname, self.path, self.validate if validate is None else bool(validate))

    def remlist(self, mapname):
        # same as __sub__ method: myList.remlist(mapname)
        remlist(mapname, self.path)

    def dellist(self):
        # clear the file: myList.dellist()
        dellist(self.path)

    # set iterable MapList object: for mapname in myList: do.myCode()
    class MapListIter(object):
        def __init__(self, maplist):
            self.maplist = maplist.getlist(maplist.validate)
            self.index = -1

        def __iter__(self):
            return self

        def next(self):
                self.index += 1
                try: return self.maplist[self.index]
                except IndexError: raise StopIteration

    def __iter__(self):
        return self.MapListIter(self)


# get list of maps from a file: maplistlib.getlist([[path] ,validate])
# if path is None, it will return all maps in ../GameName/maps/*
def getlist(path = None, validate = False):
    maplist = []
    if path is None:
        for files in os.listdir(gamedir + "/maps/"):
            filename, fileext = os.path.splitext(files)
            if fileext == ".bsp": maplist.append(filename)
    else:
        try:
            path = str(path).replace("../", "")
            openfile = open("%s/%s" % (gamedir, path), "rb")
            for line in map(lambda line: regex.sub("", line).strip(), openfile.readlines()):
                if line: maplist.append(line)
            openfile.close()
        except IOError: es.dbgmsg(0, 'maplistlib: cannot read file "%s"' % path)
    if validate: maplist = filter(lambda mapname: es.exists("map", mapname), maplist)
    return maplist

# save a new list of maps in a file: maplistlib.getlist(newList, path [,validate])
def setlist(maplist, path, validate = False):
    try:
        maplist = map(str, maplist)
        path = str(path).replace("../", "")
        openfile = open("%s/%s" % (gamedir, path), "w")
        if validate: maplist = filter(lambda mapname: es.exists("map", mapname), maplist)
        if not maplist: es.dbgmsg(1, "maplistlib: list of map is emptie")
        openfile.write("\n".join(maplist))
        openfile.close()
    except IOError: es.dbgmsg(0, 'maplistlib: cannot create file "%s"' % path)

# add a map in a file: maplistlib.addlist(mapname, path [,validate])
def addlist(mapname, path, validate = False):
    mapname = str(mapname)
    path = str(path).replace("../", "")
    if not os.path.isfile("%s/%s" % (gamedir, path)): setlist((mapname,), path, validate)
    else:
        if validate and not es.exists("map", mapname): es.dbgmsg(1, 'maplistlib: map "%s" does not exists' % mapname)
        else:
            maplist = getlist(path, False) + [mapname]
            setlist(maplist, path, False)

# remove a map from a file: maplistlib.remlist(mapname, path)
def remlist(mapname, path):
    mapname = str(mapname)
    maplist = getlist(path, False)
    if mapname in maplist:
        maplist.remove(mapname)
        setlist(maplist, path, False)
    else: es.dbgmsg(0, 'maplistlib: the map "%s" was not in list' % mapname)

# clear a file: maplistlib.dellist()
def dellist(path):
    path = str(path).replace("../", "")
    if not os.path.isfile("%s/%s" % (gamedir, path)): es.dbgmsg(0, "maplistlib: file does not exists")
    else: setlist([], path, False)
