# -*- coding:Utf-8 -*-
import es, psyco, random, gamethread, usermsg, langlib, playerlib, simplemenulib, dm_python, dm_entities, dm_language
from dm_config import dmConfig
from dm_weapon import dmWeapon
from dm_spawns import dmSpawns
psyco.full()


lastweapon = {}
dmPlayer = {}

class DmPlayer(object):
    """ Player Management """
    def __init__(self, userid):
        self.primary = None
        self.secondary = None
        self.randguns = False
        self.hegrenade = 0
        self.flashbang = 0
        self.smokegrenade = 0

        self.respawntry = 0
        self.indexspawn = -1
        self.lastcolor = None
        self.clicspawn = False
        self.firstspawn = True
        self.hasweapons = False
        self.userid = int(userid)
        self.struserid = str(userid)
        self.player = playerlib.getPlayer(userid)
        self.handle = self.player.handle
        self.lang = langlib.getLangAbbreviation(self.player.language) if self.player.language else "en"

        # protection system thread name
        self.threadProtection =  "_dm_python_protection_" + self.struserid
        # no block system thread name
        self.threadNoBlock = "_dm_python_noblock_" + self.struserid
        # respawn (try) system thread name
        self.threadRespawn = "_dm_python_respawn_" + self.struserid
        # spawn system thread name
        self.threadSpawn = "_dm_python_spawn_" + self.struserid
        # clic respawn thread name
        self.threadClic = "_dm_python_clic_" + self.struserid
        # ragdoll system thread name
        self.threadRagDoll = "_dm_python_ragdoll_" + self.struserid
        # weapons system thread name
        self.threadWeapons = "_dm_python_weapons_" + self.struserid
        # items system thread name
        self.threadItems = "_dm_python_items_" + self.struserid

        # weapons, items ans ragdoll targetname
        self.entitiesWeaponsName = "_dm_entities_weapons_" + self.struserid
        self.entitiesRagDoll = "_dm_entities_ragdolls_" + self.struserid

    def __del__(self):
        # remove entities associated to him
        if not es.exists("userid", self.userid):
            dm_entities.removePlayerRagDoll(self, delay = False)
            dm_entities.removeEntity(self.entitiesWeaponsName)

        # cancel threads
        gamethread.cancelDelayed(self.threadProtection)
        gamethread.cancelDelayed(self.threadRespawn)
        gamethread.cancelDelayed(self.threadSpawn)
        gamethread.cancelDelayed(self.threadClic)
        gamethread.cancelDelayed(self.threadRagDoll)
        gamethread.cancelDelayed(self.threadNoBlock)
        gamethread.cancelDelayed(self.threadWeapons)
        gamethread.cancelDelayed(self.threadItems)

    def __getitem__(self, name):
        # return real steamid and create infos for store last weapons used
        if name == "steamid":
            steamid = es.getplayersteamid(self.userid)
            if steamid in "BOTSTEAM_ID_PENDINGSTEAM_ID_LAN": return None
            else:
                if not lastweapon.has_key(steamid): lastweapon[steamid] = [None, None]
                return steamid

        # get last weapons used
        elif name == "lastprimary":
            steamid = self["steamid"]
            if steamid is None: return None
            else:
                try: return lastweapon[steamid][0]
                except KeyError: return None

        elif name == "lastsecondary":
            steamid = self["steamid"]
            if steamid is None: return None
            else:
                try: return lastweapon[steamid][1]
                except KeyError: return None

    def __setitem__(self, name, value):
        # set last weapons used
        if name == "lastprimary":
            steamid = self["steamid"]
            if steamid is not None:
                if lastweapon.has_key(steamid): lastweapon[steamid][0] = value

        elif name == "lastsecondary":
            steamid = self["steamid"]
            if steamid is not None:
                if lastweapon.has_key(steamid): lastweapon[steamid][1] = value

    def i_noblock(self):
        # check if noblock status can be change
        gamethread.cancelDelayed(self.threadNoBlock)
        sx, sy, sz = self.player.location
        status = 5
        for userid in es.getUseridList():
            if userid <> self.userid:
                ux, uy, uz = es.getplayerlocation(userid)
                if ((sx - ux) ** 2 + (sy - uy) ** 2 + (sz - uz) ** 2) ** 0.5 <= 150:
                    gamethread.delayedname(1, self.threadNoBlock, self.i_noblock)
                    status = 2
                    break
        es.setplayerprop(self.userid, "CBaseEntity.m_CollisionGroup", status)

    def i_protection(self):
        # set attribute by default at end of protection time
        self.player.godmode = False
        self.player.color = self.lastcolor

    def i_clicspawn(self, value):
        self.clicspawn = bool(value)

    def respawn(self):
        # respawn a player if he's dead
        if es.getplayerprop(self.userid, "CBasePlayer.pl.deadflag"):
            # determine wich plugin
            plugin = str(dmConfig.plugin).lower()
            if plugin <> "es":
                if plugin == "spe" and not dm_python.SourcePythonExtensions: plugin = "es"
                elif plugin == "nt" and not dm_python.NativeTools: plugin = "es"
            esmode = True if plugin == "es" else False

            # SPE method...
            if plugin == "spe":
                try: dm_python.es_spe.respawn(self.userid)
                except NameError:
                    es.dbgmsg(0, 'dm_python: cannot respawn userid "%s" with plugin "Source Python Extensions"' % self.userid)
                    es.dbgmsg(0, "dm_python: try to respawn the player with EventScripts only...")
                    esmode = True

            # NT method...
            elif plugin == "nt":
                try: dm_python.es_nt.spawnPlayer(self.userid)
                except NameError:
                    es.dbgmsg(0, 'dm_python: cannot respawn userid "%s" with plugin "Native Tools"' % self.userid)
                    es.dbgmsg(0, "dm_python: try to respawn the player with EventScripts only...")
                    esmode = True

            # ES method...
            if esmode: dm_entities.respawnPlayer(self)
            # if player has been spawned correctly
            es.server.insertcmd("_dp_player_respawn " + self.struserid)

    def setpos(self):
        spawn = dmSpawns.get(self.userid)
        if spawn is not None and dmSpawns.isupdated:
            ox, oy, oz, ax, ay, az, origin = spawn
            dm_entities.setPositionOfPlayer(self.userid, ox, oy, oz, ax, ay, az)

    def spawn(self):
        if not es.getplayerprop(self.userid, "CBasePlayer.pl.deadflag"):
            self.lastcolor = self.player.color
            # set position
            self.setpos()
            self.respawntry = 0
            self.hasweapons = False

            # init grenades management
            self.hegrenade = 0
            self.flashbang = 0
            self.smokegrenade = 0

            # if player want random weapon at each respawn
            if self.randguns:
                self.primary = dmWeapon.randomWeapon(True)
                self.secondary = dmWeapon.randomWeapon(False)

            # remove weapon of player
            dm_entities.stripWeaponsOfPlayer(self.userid)
            gamethread.cancelDelayed(self.threadSpawn)
            gamethread.delayedname(.1, self.threadSpawn, self._spawn)

    def _spawn(self):
        self.hasweapons = True
        # initialize noblock system
        noblock = int(dmConfig.noblock)
        if noblock == 1: es.setplayerprop(self.userid, "CBaseEntity.m_CollisionGroup", 2)
        elif noblock == 2:
            es.setplayerprop(self.userid, "CBaseEntity.m_CollisionGroup", 2)
            gamethread.cancelDelayed(self.threadNoBlock)
            gamethread.delayedname(1, self.threadNoBlock, self.i_noblock)

        # initialize protection system
        if bool(dmConfig.protection):
            self.player.godmode = True
            self.player.color = (255, 255, 255, dmConfig.alpha)
            gamethread.cancelDelayed(self.threadProtection)
            gamethread.delayedname(int(dmConfig.protection), self.threadProtection, self.i_protection)

        # give weapons and items
        gamethread.cancelDelayed(self.threadItems)
        gamethread.cancelDelayed(self.threadWeapons)
        gamethread.delayedname(.1, self.threadItems, self.items)
        gamethread.delayedname(.2, self.threadWeapons, self.weapons)

    def items(self):
        # give items and grenades
        if int(dmConfig.assaultsuit): dm_entities.giveEntityToPlayer(self.userid, "item_assaultsuit")
        if int(dmConfig.nightvision): dm_entities.giveEntityToPlayer(self.userid, "item_nvgs")
        if int(dmConfig.hegrenade): dm_entities.giveEntityToPlayer(self.userid, "weapon_hegrenade")
        if int(dmConfig.flashbang): dm_entities.giveEntityToPlayer(self.userid, "weapon_flashbang")
        if int(dmConfig.smokegrenade): dm_entities.giveEntityToPlayer(self.userid, "weapon_smokegrenade")

    def weapons(self):
        dm_entities.giveEntityToPlayer(self.userid, "weapon_knife")
        # give primary and secondary weapons to player
        if self.player.isbot:
            primary, secondary = dmWeapon.botPrimary(), dmWeapon.botSecondary()
            if primary is not None: dm_entities.giveEntityToPlayer(self.userid, "weapon_" + primary)
            if secondary is not None: dm_entities.giveEntityToPlayer(self.userid, "weapon_" + secondary)

        else:
            # try to get last weapons of player at his last connection if these ar'nt registered
            if self.primary is None:
                lastprimary = self["lastprimary"]
                if lastprimary is not None: self.primary = lastprimary
            if self.secondary is None:
                lastsecondary = self["lastsecondary"]
                if lastsecondary is not None: self.secondary = lastsecondary

            # check if weapons are allowed and select them if it was not the case
            if not self.primary in dmWeapon:
                self.primary = dmWeapon.randomWeapon(True)
                self["lastprimary"] = self.primary
            if not self.secondary in dmWeapon:
                self.secondary = dmWeapon.randomWeapon(False)
                self["lastsecondary"] = self.secondary

            # finally, give weapons if it could be
            if self.secondary: dm_entities.giveEntityToPlayer(self.userid, "weapon_" + self.secondary)
            if self.primary: dm_entities.giveEntityToPlayer(self.userid, "weapon_" + self.primary)

    def grenade(self, name):
        # give a grenade if it remains
        if not es.getplayerprop(self.userid, "CBasePlayer.pl.deadflag"):
            if name == "he":
                self.hegrenade += 1
                if self.hegrenade < int(dmConfig.hegrenade): dm_entities.giveEntityToPlayer(self.userid, "weapon_hegrenade")
            elif name == "fb":
                self.flashbang += 1
                if self.flashbang < int(dmConfig.flashbang): dm_entities.giveEntityToPlayer(self.userid, "weapon_flashbang")
            elif name == "sg":
                self.smokegrenade += 1
                if self.smokegrenade < int(dmConfig.smokegrenade): dm_entities.giveEntityToPlayer(self.userid, "weapon_smokegrenade")

    def death(self):
        # delete weapons, items, ragdolls, etc...
        time = float(dmConfig.respawn)
        clic = float(dmConfig.clicspawn)
        bot = float(dmConfig.botRespawn)

        self.clicspawn = False
        try: self.player.color = 255, 255, 255, 255
        except TypeError: pass
        dm_entities.removeEntity(self.entitiesWeaponsName, self.userid)
        dm_entities.removePlayerRagDoll(self, bool(dmConfig.dissolver))

        # respawn player if round is not over
        if dm_python.roundstat:
            gamethread.cancelDelayed(self.threadRespawn)
            if not self.player.isbot: gamethread.delayedname(time if time >= 1 else 1, self.threadRespawn, self.respawn)
            else: gamethread.delayedname(bot if bot >= 1 else 1, self.threadRespawn, self.respawn)

            if clic >= 1:
                # set observer mode
                gamethread.cancelDelayed(self.threadClic)
                gamethread.delayedname(clic if clic >= 1 else 1, self.threadClic, self.i_clicspawn, True)
                if not self.player.isbot: es.setplayerprop(self.userid, "CBasePlayer.m_iObserverMode", 6)
                usermsg.hudhint(self.userid, dm_language.message(self.userid, "deathmatch death hud", {"second" : dmConfig.respawn}))


# Function for get/manage players
def player(userid):
    userid = int(userid)
    if not dmPlayer.has_key(userid): dmPlayer[userid] = DmPlayer(userid)
    return dmPlayer[userid]

def playerlist(filter = "all"):
    if filter == "all": return dmPlayer.keys()
    elif filter == "human": return [userid for userid in playerlist() if not dmPlayer[userid].isbot]

def unload():
    if bool(dmConfig.protection):
        for player in dmPlayer.values():
            if not es.getplayerprop(player.userid, "CBasePlayer.pl.deadflag"):
                player.godmode = False
                player.color = player.lastcolor
    lastweapon.clear()
    dmPlayer.clear()
